/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Insets;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.JToolTip;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.filechooser.FileFilter;
import javax.swing.tree.DefaultMutableTreeNode;

import net.sf.dc.console.components.DcFileField;
import net.sf.dc.console.components.DcFontSelector;
import net.sf.dc.console.components.DcList;
import net.sf.dc.console.components.DcLongTextField;
import net.sf.dc.console.components.DcMultiLineToolTip;
import net.sf.dc.console.components.DcNumberField;
import net.sf.dc.console.components.DcPasswordField;
import net.sf.dc.console.components.DcPictureField;
import net.sf.dc.console.components.DcProgramDefinitionsField;
import net.sf.dc.console.components.DcRatingComboBox;
import net.sf.dc.console.components.DcReportTextPositionCombo;
import net.sf.dc.console.components.DcShortTextField;
import net.sf.dc.console.components.DcTimeField;
import net.sf.dc.console.components.DcUrlField;
import net.sf.dc.console.components.renderers.ComboBoxRenderer;
import net.sf.dc.console.components.tables.DcTable;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.cache.CacheManager;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.jflexsett.settings.ui.ISettingsComponent;

import com.digitprop.tonic.TonicLookAndFeel;

/**
 * This class is the Component holder for the UI. The look and feel, as well
 * as all the other preference are set here
 *
 * @author Robert Jan van der Waals
 * @since 1.3
 * @version 0.5
 */
public final class ComponentFactory {

    public static final int _SOFTWARECATEGORYCOMBO = 0;
    public static final int _YESNOCOMBO = 1;
    public static final int _MP3GENRECOMBO = 2;
    public static final int _LONGTEXTFIELD = 3;
    public static final int _YEARCOMBO = 5;
    public static final int _CHECKBOX = 6;
    public static final int _NUMBERFIELD = 7;
    public static final int _SHORTTEXTFIELD = 8;
    public static final int _URLFIELD = 9;
    public static final int _PICTUREFIELD = 10;
    public static final int _FONTSELECTOR = 11;
    public static final int _TIMEFIELD = 12;
    public static final int _THEMEFIELD = 13;
    public static final int _RATINGCOMBOBOX = 14;    
    public static final int _FILEFIELD = 15;
    public static final int _PASSWORDFIELD = 16;
    public static final int _BOOKCATEGORYCOMBO = 17;
    public static final int _REPORTTEXTPOSCOMBO = 18;
    public static final int _PROGRAMDEFINITIONFIELD = 19;
    
    private static Font fontUnreadable = new Font("Dialog", Font.PLAIN, 1);
    private static final Color colorDisabled = new Color(240,240,240);
    private static final Color colorRequired = new Color(120, 0, 0);
    
    private static javax.swing.plaf.ButtonUI checkBoxUI;
    
    static {
        try {
            // fix for the check box UI of the look and feel
            checkBoxUI = new JCheckBox().getUI();
            // set the look and feel
            UIManager.setLookAndFeel(new TonicLookAndFeel());
        } catch (Exception e) {
            Messages.add(e);
        }
    }

    public static final JComponent getComponent(  int componentIndex,
                                                  String sLabel,
												  int maxTextLength) {
        switch (componentIndex) {
            case _SOFTWARECATEGORYCOMBO:
                return getSoftwareCatComboBox();
            case _YESNOCOMBO:
                return getYesNoCombo();
            case _MP3GENRECOMBO:
                return getMP3GenreComboBox();
            case _LONGTEXTFIELD:
                return getLongTextField(maxTextLength);
            case _YEARCOMBO:
                return getYearComboBox();
            case _CHECKBOX:
                return getCheckBox(sLabel);
            case _NUMBERFIELD:
                return getNumberField();
            case _URLFIELD:
                return getURLField(maxTextLength);
            case _PICTUREFIELD:
                return getPictureField(true, true, false);
            case _FONTSELECTOR:
                return getFontSelector();
            case _TIMEFIELD:
                return getTimeField();
            case _RATINGCOMBOBOX:
                return getRatingComboBox();
            case _FILEFIELD:
                return getFileField(false, false);
            case _PASSWORDFIELD:
                return getPasswordField();
            case _BOOKCATEGORYCOMBO:
                return getBookCatComboBox();
            case _REPORTTEXTPOSCOMBO:
                return getReportTextPositionCombo();
            case _PROGRAMDEFINITIONFIELD:
                return getProgramDefinitionField();
            default:
                return getShortTextField(maxTextLength);
        }
    }

	public static final JComponent setUneditable(JComponent comp) {
		if (comp instanceof JComboBox)
			 ((JComboBox)comp).setEnabled(false);
		else if (comp instanceof JTextField)
			((JTextField) comp).setEditable(false);
		
		return comp;
	}
    
	public static final DcProgramDefinitionsField getProgramDefinitionField() {
		return new DcProgramDefinitionsField();
	}
	
    public static final DcReportTextPositionCombo getReportTextPositionCombo() {
        return new DcReportTextPositionCombo();
    }
    
    public static final DcPasswordField getPasswordField() {
    	return new DcPasswordField();
    }
	
    public static final DcFontSelector getFontSelector() {
        return new DcFontSelector(); 
    }
    
    public static final DcTimeField getTimeField() {
    	return new DcTimeField();
    }
    
    public static final DcFileField getFileField(boolean save, boolean dirsOnly) {
    	return new DcFileField(save, dirsOnly);
    }
    
    public static final DcFileField getFileField(boolean save, boolean dirsOnly, FileFilter filter) {
    	return new DcFileField(save,  dirsOnly, filter);
    }

    public static final DcList getList(Collection data) {
        DcList list = new DcList(data);
        list.setFont(getStandardFont());
        return list;
    }
    
    public static final DcList getList() {
        DcList list = new DcList();
        list.setFont(getStandardFont());
        return list;
    }

    public static final DcPictureField getPictureField(boolean scaled, 
                                                     boolean allowZoom, 
                                                     boolean thumbnail) {
        DcPictureField pictureField = new DcPictureField(scaled, allowZoom, thumbnail) {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        pictureField.setBorder(getBorder());
        return pictureField;
    }    
    
    public static final DcPictureField getPictureField(boolean allowZoom) {
        DcPictureField pictureField = new DcPictureField(allowZoom) {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        pictureField.setBorder(getBorder());
        return pictureField;
    }

    public static final DcUrlField getURLField(int maxLength) {
        DcUrlField urlField = new DcUrlField(maxLength);
        urlField.setBorder(getBorder());
        urlField.setFont(getStandardFont());
        return urlField;
    }

    public static final JColorChooser getColorChooser() {
        JColorChooser colorChooser = new JColorChooser();
        colorChooser.setFont(getStandardFont());
        return colorChooser;
    }

    public static final JTree getTree(DefaultMutableTreeNode model) {
        JTree tree = new JTree(model);
        tree.setFont(getStandardFont());
        return tree;
    }

    public static final DcNumberField getNumberField() {
        DcNumberField numberField = new DcNumberField();
        numberField.setFont(getStandardFont());
        numberField.setBorder(getBorder());
        return numberField;
    }
    
    public static final DcRatingComboBox getRatingComboBox() {
    	DcRatingComboBox combo = new DcRatingComboBox();
        return combo;
    }

    /**
     * Returns a combobox with the Software database field names
     * @return (JComboBox)  combobox filled with the database field names.
     *                      The first item in the list is an empty String
     */
    public static final JComboBox getFieldsComboBox(DcObject o) {
        JComboBox fieldsComboBox = getComboBox();
        fieldsComboBox.addItem(" ");
        
        String[] fields = o.getDatabaseFieldNames();
        for (int i = 0; i < fields.length; i++) {
            fieldsComboBox.addItem(fields[i]);
        }
        fieldsComboBox.setFont(getStandardFont());

        return fieldsComboBox;
    }

    public static final JComboBox getYearComboBox() {
        JComboBox yearComboBox = getComboBox();
        yearComboBox.addItem("");
        Date date = new Date();
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        int year = cal.get(Calendar.YEAR);

        for (int i = year; i > 1939; i--) {
            yearComboBox.addItem(String.valueOf(i));
        }

        return yearComboBox;
    }
    
    /**
     * Returns a combobox with MP3 genres
     * @return combobox filled with MP3 genres. The first item is
     *         an empty String
     */
    public static final JComboBox getMP3GenreComboBox() {
        JComboBox genreComboBox = getComboBox();
        genreComboBox.addItem("");
        genreComboBox.setSelectedIndex(0);
        genreComboBox.setFont(getStandardFont());

        for (int i = 0; i < Repository.collections.colMusicGenres.length; i++) {
            genreComboBox.addItem(Repository.collections.colMusicGenres[i]);
        }

        return genreComboBox;
    }

	public static final JComboBox getSoftwareCatComboBox() {
        JComboBox comboBox = getComboBox();
        CacheManager.registerUIComponent(comboBox, CacheManager._SOFTWARECATEGORY);
		return comboBox;
	}
    
    public static final JComboBox getBookCatComboBox() {
        JComboBox comboBox = getComboBox();
        CacheManager.registerUIComponent(comboBox, CacheManager._BOOKCATEGORY);
        return comboBox;
    }    
    
    /**
     * Returns an empty combobox
     *
     * @param nameOfField   name of this field
     * @return (JComboBox)  an empty combobox
     */
    public static final JComboBox getComboBox() {
        JComboBox comboBox = new JComboBox();
        comboBox.setFont(getStandardFont());
        comboBox.setBorder(getBorder());
        comboBox.setRenderer(new ComboBoxRenderer());
        return comboBox;
    }

    /**
     * Returns a combobox with the items "yes" and "no", not starting with
     * and empty String
     *
     * @return (JComboBox)  combobox with the items "yes" and "no"
     */
    public static final JComboBox getRealYesNoComboBox() {
        JComboBox comboBox = getComboBox();
        comboBox.addItem(DcResources.getText(Repository.labels.lblNo));
        comboBox.addItem(DcResources.getText(Repository.labels.lblYes));
        comboBox.setFont(getStandardFont());
        return comboBox;
    }
    
    /**
     * Returns a standard checkbox
     *
     * @param nameOfField   name of this field
     * @return (JCheckBox)  simple checbox, no prefenrences are set
     */
    public static final JCheckBox getCheckBox(String labelText) {
        JCheckBox checkBox = new JCheckBox(labelText);
        checkBox.setUI(checkBoxUI);
        checkBox.setFont(getSystemFont());
        checkBox.setForeground(Color.GRAY);
        return checkBox;
    }

    public static final JMenuItem getMenuItem(String text) {
        JMenuItem menuItem = new JMenuItem(text);
        menuItem.setFont(getSystemFont());
        return menuItem;
    }    
    
    public static final JMenuItem getMenuItem(ImageIcon icon, String text) {
    	JMenuItem menuItem = getMenuItem(text);
        menuItem.setIcon(icon);
        return menuItem;
    }
    
    public static final JMenu getMenu(String text) {
        JMenu menu = new JMenu(text);
        menu.setFont(getSystemFont());
        return menu;
    }
    
    /**
     * Returns a standard button
     *
     * @param buttonText    display String, also used for naming the button
     * @param length        length of this button
     * @return (JButton)    button, height 23
     */
    public static final JButton getTableHeader(String title) {
        JButton button = getButton(title);
        button.setFont(ComponentFactory.getStandardFont());
        button.setPreferredSize(new Dimension(button.getWidth(), 20));
        button.setBorder(new EtchedBorder(5));
        return button;
    }    

    /**
     * Returns a standard button
     *
     * @param buttonText    display String, also used for naming the button
     * @param length        length of this button
     * @return (JButton)    button, height 23
     */
    public static final JButton getButton(String buttonText, int length) {
        JButton button = getButton(buttonText);
        button.setPreferredSize(new Dimension(length, 23));
        return button;
    }

    /**
     * Returns a standard button
     *
     * @param buttonText    display String, also used for naming the button
     * @return (JButton)    button, height 23
     */
    public static final JButton getButton(ImageIcon icon) {
        JButton button = new JButton("", icon) {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        button.setFont(getSystemFont());
        return button;
    }
    
    /**
     * Returns a standard button
     *
     * @param buttonText    display String, also used for naming the button
     * @return (JButton)    button, height 23
     */
    public static final JButton getButton(ImageIcon icon, String buttonText) {
        JButton button = new JButton(buttonText, icon) {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        button.setFont(getSystemFont());
        return button;
    }    

    /**
     * Returns a standard button
     *
     * @param buttonText    display String, also used for naming the button
     * @return (JButton)    button, height 23
     */
    public static final JButton getButton(String buttonText) {
        JButton button = new JButton() {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        button.setName("bt" + buttonText);
        button.setText(buttonText);
        button.setToolTipText(buttonText);
        button.setFont(getSystemFont());
        return button;
    }

    /**
     * Returns a long text field with all the defaults set
     */
    public static final JTextArea getTextArea() {
        JTextArea textArea = new JTextArea();
        textArea.setWrapStyleWord(true);
        textArea.setLineWrap(true);
        textArea.setEditable(true);
        textArea.setDoubleBuffered(true);
        //textArea.setBackground(getTableColor());
        textArea.setMargin(new Insets(5,5,5,5));
        textArea.setFont(getStandardFont());

        return textArea;
    }
    
    public static final JTextPane getTextPane() {
        JTextPane textpane = new JTextPane();
        return textpane;
    }

    /**
     * Returns a text area with all the defaults set
     */
    public static final DcLongTextField getLongTextField(int maxTextLength) {
        DcLongTextField longText = new DcLongTextField(maxTextLength);
        longText.setWrapStyleWord(true);
        longText.setLineWrap(true);
        longText.setEditable(true);
        longText.setDoubleBuffered(true);
        //longText.setBackground(getTableColor());
        longText.setMargin(new Insets(5,5,5,5));
        longText.setFont(getStandardFont());

        return longText;
    }

    /**
     * Returns a standard label with an icon
     * @param icon 
     */
    public static final JLabel getLabel(ImageIcon icon) {
        JLabel label = new JLabel(icon);
        label.setForeground(Color.GRAY);
        return label;
    }     
    
    /**
     * Returns a standard label with a text and an icon
     * @param length        length of this field
     * @param labeltext     display String
     * @return (JLabel)     label, no preferemces are set
     */
    public static final JLabel getLabel(String labelText, ImageIcon icon) {
        JLabel label = new JLabel(labelText);
        label.setIcon(icon);
        label.setForeground(Color.GRAY);
        return label;
    }    
    
    /**
     * Returns a standard label with a text
     * @param length        length of this field
     * @param labeltext     display String
     * @return (JLabel)     label, no preferemces are set
     */
    public static final  JLabel getLabel(String labelText, int length) {
        JLabel label = getLabel(labelText);
        label.setPreferredSize(new Dimension(length, 23));
        label.setText(labelText);
        
        label.setForeground(Color.GRAY);
        
        return label;
    }

    /**
     * Returns a standard label with a text
     *
     * @param labeltext     display String
     * @return (JLabel)     label, no preferemces are set
     */
    public static final JLabel getLabel(String labelText) {
        JLabel label = new JLabel() {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        
        label.setForeground(Color.GRAY);
        
        label.setText(labelText);
        label.setRequestFocusEnabled(false);
        label.setFont(getSystemFont());
        label.setToolTipText(labelText); 
        
        return label;
    }

    /**
     * Returns a standard editor pane
     * @return editor pane with no preferences are set
     */
    public static final JEditorPane getEditorPane() {
        JEditorPane editorPane = new JEditorPane();
        editorPane.setFont(getStandardFont());
        return editorPane;
    }

    public static final DcShortTextField getShortTextField(int maxTextLength) {
        DcShortTextField textField = new DcShortTextField(maxTextLength);
        textField.setFont(getStandardFont());
        textField.setPreferredSize(new Dimension(50, 20));
        textField.setBorder(getBorder());
        return textField;
    }

    public static final JTextField getTextFieldDisabled() {
        JTextField textField = new JTextField();
        textField.setPreferredSize(new Dimension(50, 20));
        textField.setBorder(getBorder());
        textField.setEnabled(false);
        textField.setFont(ComponentFactory.getSystemFont());
        textField.setForeground(ComponentFactory.getDisabledColor());        
        
        return textField;
    }
    
    /**
     * Returns a textfield
     *
     * @return JTextField   textfield, height 23
     */
    public static final JTextField getTextField() {
        JTextField textField = new JTextField();
        textField.setFont(getStandardFont());
        textField.setPreferredSize(new Dimension(50, 20));
        textField.setBorder(getBorder());
        return textField;
    }

    /**
     * Returns a standard tabbed pane
     */
    public static final JTabbedPane getTabbedPane() {
        JTabbedPane tabbedPane = new JTabbedPane();
        tabbedPane.setFont(getSystemFont());
        return tabbedPane;
    }

    public static final DcTable getDCTable(boolean readonly, boolean caching) {
        DcTable table = new DcTable(readonly, caching) {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        return table;
    }
    
    public static final DcTable getDCTable(DCModule module, boolean readonly, boolean caching) {
    	DcTable table = new DcTable(module, readonly, caching) {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        return table;
    }
    
    /**
     * Returns a standard icon
     * @param iconLocation  location of the icon image
     */
    public static final Icon getIcon(String iconLocation) {
        Icon icon = new ImageIcon(iconLocation);
        return icon;
    }

    /**
     * Returns a standard menubar
     * @param nameOfField   name of this field
     */
    public static final JMenuBar getMenuBar() {
        JMenuBar menuBar = new JMenuBar() {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        menuBar.setFont(getStandardFont());
        return menuBar;
    }

    /**
     * Returns a combobox with the items " ", "yes" and "no"
     *
     * @param nameOfField   name of this field
     * @return (JComboBox)  a combox with the items "yes" and "no", the first
     *                      item is an empty String
     */
    public static final JComboBox getYesNoCombo() {
        JComboBox comboBox = getComboBox();
        comboBox.addItem("");
        comboBox.addItem(DcResources.getText(Repository.labels.lblYes));
        comboBox.addItem(DcResources.getText(Repository.labels.lblNo));
        comboBox.setFont(getStandardFont());

        return comboBox;
    }
    
    /**
     * Returns a standard border
     */
    public static final Border getBorder() {
        return BorderFactory.createLineBorder(Color.GRAY);
    }    

    /**
     * Returns a standard border with a title
     * @param title             display String
     */
    public static final TitledBorder getTitleBorder(String title) {
        TitledBorder border = new TitledBorder(
                BorderFactory.createLineBorder(Color.LIGHT_GRAY, 1), 
                title);
        border.setTitleFont(getSystemFont());
        //border.setTitleColor(getSelectionColor());
        return border;
    }

    public static final Font getStandardFont() {
        return DcSettings.getValueAsFont(Repository.settings.stSystemFontNormal);
    }

    public static final Font getSystemFont() {
        return DcSettings.getValueAsFont(Repository.settings.stSystemFontBold); 
    }
    
    public static final Font getUnreadableFont() {
        return fontUnreadable;
    }

    /**
     * Returns the color for displaying disabled fields
     */
    public static final Color getDisabledColor() {
       return colorDisabled;
    }

    /**
     * Returns the color for displaying required fields
     */
    public static final Color getRequiredColor() {
        return colorRequired;
    }

    /**
     * Returns the color for the table header
     */
    public static final Color getTableHeaderColor() {
        return new Color(220, 220, 220);
    }

    public static final void setComponentValue(int fieldType, JComponent comp, Object value) {
        switch (fieldType) {
            case (ComponentFactory._SOFTWARECATEGORYCOMBO):
            case (ComponentFactory._BOOKCATEGORYCOMBO):
            case (ComponentFactory._YESNOCOMBO):
            case (ComponentFactory._MP3GENRECOMBO):
            case (ComponentFactory._YEARCOMBO):
            case (ComponentFactory._RATINGCOMBOBOX):
                ((JComboBox) comp).setSelectedItem(value);
                break;
            case (ComponentFactory._REPORTTEXTPOSCOMBO):
                ((DcReportTextPositionCombo) comp).setValue(value); 
                break;
            case (ComponentFactory._NUMBERFIELD):
                ((DcNumberField) comp).setValue(value);
                break;
            case (ComponentFactory._LONGTEXTFIELD):
                ((DcLongTextField) comp).setValue(value);
                break;
            case (ComponentFactory._PICTUREFIELD):
                ((DcPictureField) comp).setValue(value);
                break;
            case (ComponentFactory._TIMEFIELD):
                ((DcTimeField) comp).setValue(value);
                break;
            case (ComponentFactory._PROGRAMDEFINITIONFIELD):
                ((ISettingsComponent) comp).setValue(value);
                break;
            default:
                ((JTextField) comp).setText(value.toString());
        }
    }

    public static Object getComponentValue(int fieldType, JComponent comp) {
        switch (fieldType) {
            case (ComponentFactory._SOFTWARECATEGORYCOMBO):      
            case (ComponentFactory._BOOKCATEGORYCOMBO):
            case (ComponentFactory._YESNOCOMBO):
            case (ComponentFactory._MP3GENRECOMBO):
            case (ComponentFactory._YEARCOMBO):
                Object o = ((JComboBox) comp).getSelectedItem();
                if (o == null) {
                    return "";
                } else {
                    return o.toString();
                }
            case (ComponentFactory._REPORTTEXTPOSCOMBO) :
                return ((DcReportTextPositionCombo) comp).getValue();
            case (ComponentFactory._RATINGCOMBOBOX):
                return ((DcRatingComboBox) comp).getValue();
            case (ComponentFactory._NUMBERFIELD):
                return ((DcNumberField) comp).getValue();
            case (ComponentFactory._LONGTEXTFIELD):
                return ((DcLongTextField) comp).getValue();
            case (ComponentFactory._PICTUREFIELD):
                return ((DcPictureField) comp).getValue();
            case (ComponentFactory._TIMEFIELD):
                return ((DcTimeField) comp).getValue();
            case (ComponentFactory._PROGRAMDEFINITIONFIELD):
                return ((ISettingsComponent) comp).getValue();
            default:
                return ((JTextField) comp).getText();
        }
    }
}
