/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.audiocd;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JComponent;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.panels.TableInsertPanel;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;

/**
 * The insert Audio Track panel. This panel is different from the default panels
 * as this is a child panel of the Audio CD Panel
 * @see net.sf.dc.console.audiocd.InsertAudioCDPanel
 * 
 * @author Robert-Jan van der Waals
 * @since 1.4
 * @version 1.50
 */
public class InsertAudioTrackPanel extends TableInsertPanel {

    private ArrayList llTracks = new ArrayList();
    
    /**
     * Creates the panel
     */
    public InsertAudioTrackPanel() {
        super(DCModules.getModule(DCModules._AUDIOTRACK),
              DcResources.getText(Repository.labels.lblNewItem, 
                                  DcResources.getText(Repository.systemlabels.sysAudioTrack)), 
              IconLibrary._icoNew);
    }
    
    
    public void setParentID(String sID) {
        cacheChildren();
        
        if (sID != null && !sID.equals(parentID)) {
            parentID = sID;
            loadChildren();   
        }
    }
    
    /**
     * Clears the UI table, does not clear the cache of changed values
     */
    public void clearTable() {
        clearTable(false);
    }    
    
    /**
     * Opens the selected item
     */
    public void openItemForm() {
        cancelEdit();
        
        int selectedRow = table.getSelectedRow();
        if (selectedRow == -1) {
            new MessageBox(DcResources.getText(Repository.messages.msgSelectRowToOpen),
                           MessageBox._WARNING);
        } else {
            DcObject dco = getObjectForSaving(selectedRow);
            dco.removeRequestors();
            openItemForm(dco, true);
        }
    }        
    
    public void removeRows(String[] sIDs) {
    	super.removeRows(sIDs);
        for (Iterator iter = llTracks.iterator(); iter.hasNext(); ) {
            DcObject dco = (DcObject) iter.next();
        	for (int i = 0; i < sIDs.length; i++) {
                if (dco.getID().equals(sIDs[i])) {
                	llTracks.remove(dco);
                }
        	}
        }
    }
    
    /**
     * Clears the UI table
     * @param cache clear cache as well ?
     */
    public void clearTable(boolean cache) {
    	table.clearTable();
        if (cache) {
        	llTracks.clear();
        }
    }
    
    /**
     * Store the newly added children in a hashtable so they can be showed to
     * the user again when the user selects the parent
     */
    public void cacheChildren() {
        Collection children = getChildren(getParentID());
        llTracks.removeAll(children);
        
        for (int i = 0; i < table.getRowCount(); i++) {
        	DcObject dco = getObjectForSaving(i);
            dco.setValue(dco.getParentReferenceFieldIndex(), getParentID());
            
            boolean exists = false;
            for (Iterator iter = llTracks.iterator(); iter.hasNext(); ) {
            	DcObject dcoCached = (DcObject) iter.next();
                if (dcoCached.getID().equals(dco.getID())) {
                	exists = true;
                    break;
                }
            }
            
            if (!exists) {
            	llTracks.add(dco);
            }
        }
    }
    
    /**
     * Removes children from both the UI table and the cache
     * @param parentID the ID of the parent to which the children belong 
     */
    public void removeChildren(String parentID) {
        Collection children = getChildren(parentID);
        llTracks.removeAll(children);
        clearTable(false);
    }
    
    /**
     * Loads the children from the cache
     * @param parentID the ID of the parent for which the children are loaded
     */
    public void loadChildren() {
        clearTable();
        Collection children = getChildren(parentID);
        for (Iterator iter = children.iterator(); iter.hasNext(); ) {
            DcObject child = (DcObject) iter.next();
            child.setValue(child.getParentReferenceFieldIndex(), 
                           parentID);
            addObject(child);
        }
    }    
    
    /**
     * Retrieves the children for a parent
     * @param parentID the ID of the parent for which the children are retrieved
     */
    public Collection getChildren(String parentID) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < llTracks.size(); i++) {
            DcObject dco = (DcObject) llTracks.get(i);
            if (dco.getValue(dco.getParentReferenceFieldIndex()).equals(parentID) ) {
                al.add(dco);
            }
        }
        return al;
    }
    
    /**
     * Return extra action buttons
     */
    public JComponent[] getExtraComponents() {
        JButton buttonAddTrack = ComponentFactory.getButton(DcResources.getText(Repository.labels.lblAddTrack));
        buttonAddTrack.addActionListener(new AddTrackAcionListener());
        return new JComponent[] {buttonAddTrack};
    }
    
    /**
     * Adds a track (child) to the selected audio CD (parent) 
     */
    private class AddTrackAcionListener implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            if (getParentID() != null && !getParentID().equals("")) {
                DcObject dco = getModule().getDCObject();
                dco.setValue(dco.getParentReferenceFieldIndex(), getParentID());
                addObject(dco);
            } else {
                new MessageBox(DcResources.getText(Repository.messages.msgAddSelectAlbum), 
                               MessageBox._ERROR);
            } 
        }
    }     
}
