/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.panels;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Collection;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.tables.DcTable;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.forms.ItemForm;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;

/**
 * Shows a table with (readonly) child objects. The objects can be opened
 * but cannot be modified.
 *
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.5
 */
public class ChildListPanel extends JPanel {

    private final DCModule module;
    private final DcTable table;

    /**
     * Initializes this component. 
     * @param module the child module
     */
    public ChildListPanel(DCModule module) {
        this.module = module;
        this.setBackground(DcSettings.getValueAsColor(Repository.settings.stColor));
        table = ComponentFactory.getDCTable(module, true, false);
        table.addMouseListener(new TableMouseListener());
        buildComponent();
    }
    
    public void setChildren(Collection children) {
        table.clearTable();
        fillTable(children);
    }
    
    public void flush() {
        if (table != null) {
            table.clearTable();   
        }
    }
    
    /**
     * Returns all children
     */
    public DcObject[] getChildren() {
        if (table != null) {
            return table.getDCObjects();    
        } else {
        	return new DcObject[0];
        }
    }
    
    /**
     * Fills the table with the specified child objects
     * @param children array of child objects
     */
    private void fillTable(Collection children) {
        table.applyDefinitions();
        table.addObjects(children);
    }
    
    /**
     * Opens the selected row in an item form (readonly)
     */
    private void openChildItem() {
        int selectedRow = table.getSelectedRow();
        if (selectedRow == -1) {
            new MessageBox(DcResources.getText(Repository.messages.msgNoRowsSelected), MessageBox._WARNING);
        	return;
        }
        DcObject dco = table.getDCObject(selectedRow);
        
        ItemForm itemForm = new ItemForm(  null,
                                           true, 
                                           true, 
                                           dco, 
                                           module.isAmazonUpdateAllowed(), 
                                           module.isImdbUpdateAllowed(),
                                           module.getSearchPanel().getClass(), 
                                           dco.getRow());      
        itemForm.setVisible(true);
    }

    private void buildComponent() {
        this.setLayout(Layout.getGBL());
        
        JScrollPane scrollPane = new JScrollPane(table);
        
        JPanel panelActions = new JPanel();
        panelActions.setBackground(DcSettings.getValueAsColor(Repository.settings.stColor));
        panelActions.setLayout(Layout.getGBL());
        
        JButton buttonOpen = ComponentFactory.getButton(
                        DcResources.getText(Repository.labels.lblViewItem, module.getObjectName()));
        buttonOpen.addActionListener(new OpenChildAction());
        
        panelActions.add(buttonOpen, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                         new Insets(5, 5, 5, 5), 0, 0));

        this.add(scrollPane,  Layout.getGBC( 0, 0, 1, 1, 10.0, 10.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets(5, 5, 5, 5), 0, 0)); 
        this.add(panelActions,  Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets(0, 0, 0, 0), 0, 0)); 
        
    }    
    
    private class OpenChildAction implements ActionListener {
    	public void actionPerformed(ActionEvent arg0) {
            openChildItem();
    	}
    }
    
    private class TableMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                openChildItem();
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }    
}
