/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.panels;

import java.awt.Cursor;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ToolTipManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.MainFrame;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.SaveQuestionBox;
import net.sf.dc.console.reporting.ReportingDialog;
import net.sf.dc.console.software.SoftwareCategoryForm;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.processes.DataTask;
import net.sf.dc.resources.DcResources;
import net.sf.dc.wf.requests.RemoveRowsRequest;
import net.sf.dc.wf.requests.StatusUpdateRequest;

/**
 * Base for all insert panels (for objects like Software, Music and Movie)
 * Has an implemenation for supporting a child panel (in case of an Object
 * with children). The functionality is not fully implemented in this class
 * as only (for now) the Audio Track and Audio CD panel have a parent-child
 * relation. 
 *
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.6
 */
public abstract class TableInsertPanel extends TablePanel {
    
    protected String parentID = null;
    protected TableInsertPanel childPanel;

    /**
     * Initializes the panel. Should be called by the child by calling
     * super(module). The popup menu and the action panel are automatically
     * created. 
     * @param module the module for which this panel is created
     * @param title the title of this panel
     * @param icon the icon of this panel
     */
    public TableInsertPanel (DCModule module, String title, ImageIcon icon) {
        super(module, title, icon);  
        setHelpIndex("dc.Inserting");
        
        boolean imdb = getModule().isImdbUpdateAllowed();
        boolean amazon = getModule().isAmazonUpdateAllowed();
        boolean open = this instanceof SoftwareCategoryForm.CatagoryInsertPanel ?
                       false : true;
        
        JComponent[] extraComponents = getAllExtraComponents();
        
        panelTableActions = new TableActionsPanel(true, extraComponents);
        panelTableActions.addSaveActionListener(new SaveNewValuesAction());
            
        popupMenu = new PopupMenu(imdb, amazon, false, true, open, false);
        popupMenu.addAmazonActionListener(new AmazonUpdateAction());
        popupMenu.addIMDBActionListener(new ImdbUpdateAction());
        popupMenu.addSaveActionListener(new SaveSelectedValuesAction());
        
        buildPanel();
    }
    
    public void addObject(DcObject dco) {
        if (dco.getID() == null || dco.getID().equals("")) {
            dco.setIDs();
        }
        table.addObject(dco);
        
        if (hasChild()) {
        	childPanel.addObjects(dco.getChildren());
            childPanel.setParentID(dco.getID());
        }
    }    
    
    /**
     * Adds a list of data crow objects to the table. Mostly this
     * is used by internet updates (retrieving information from another source). 
     * The data added is ready for saving. In case the object has children, 
     * these are added to this table as well.
     * @param data list of data crow objects
     * @param addUIRowID display the UI row ID
     */
    public void addObjects(Collection objects) {
        for (Iterator iter = objects.iterator(); iter.hasNext(); ) {
        	DcObject dco = (DcObject) iter.next();
            table.addObject(dco);
        }
    }
    
    /**
     * Opens the selected item
     */
    public void openItemForm() {
        cancelEdit();
        
        int selectedRow = table.getSelectedRow();
        if (selectedRow == -1) {
            new MessageBox(DcResources.getText(Repository.messages.msgSelectRowToOpen),
                           MessageBox._WARNING);
        } else {
        	DcObject dco = getObjectForSaving(selectedRow);
            dco.removeRequestors();
            openItemForm(dco, false);
        }
    }    
    
    /**
     * Creates a PDF report. All visible fields are reported on.
     * A file browser is opened in which the user can indicate where
     * the pdf file should be stored.
     */
    public void createPdfReport() {
        DcObject[] objects = new DcObject[table.getRowCount()];
        for (int i = 0; i < objects.length; i++) {
            objects[i] = getObjectForSaving(i); 
        }
        
        ReportingDialog dialog = new ReportingDialog(objects);
        dialog.setVisible(true);
    } 
    
    /**
     * Set the parent ID (if this panel is acting as a child)
     * @param sID parent ID
     */
    public void setParentID(String sID) {}
    
    /**
     * Clear this table and the child table
     */
    public void clearTable() {
        super.clearTable();
        
        if (hasChild()) {
            childPanel.clearTable(true);
        }
    } 
    
    public void clearTable(boolean cache) {}
    
    /**
     * Retrieve module specific components
     */
    private JComponent[] getAllExtraComponents() {
        JComponent[] components = new JComponent[0];
        if (hasChild()) {
            JComponent[] childComponents = getChildModule().getInsertPanel().getExtraComponents();
            JComponent[] parentComponents = getExtraComponents();
            
            components = new JComponent[parentComponents.length + childComponents.length];
            int counter = 0; 
            for (int i = 0; i < childComponents.length; i++) {
                components[counter] = childComponents[i];
                counter++;
            }
            for (int i = 0; i < parentComponents.length; i++) {
                components[counter] = parentComponents[i];
                counter++;
            }
        }
        return components;
    }

    /**
     * Retrieves a data crow object for the specified row for saving purposes.
     * Children are added to this object
     * @param row row number
     */
    public DcObject getObjectForSaving(int row) {
        DcObject dco = table.getDCObject(row);
        
        dco.setBatch(true);
        dco.setSilence(true);
        dco.setRow(row);
        dco.setValue(DcObject._ID, dco.getID());

        dco.addRequest(new RemoveRowsRequest(
                              dco.getModule().getIndex(),
                              MainFrame._INSERTTAB,
							  new String[] {dco.getID()}));
        
        if (hasChild()) {
            childPanel.setParentID(dco.getID());
            Collection children = childPanel.getChildren(dco.getID());
            
            for (Iterator iter = children.iterator(); iter.hasNext(); ) {
            	DcObject child = (DcObject) iter.next();
                child.addRequest(new RemoveRowsRequest(
                        childPanel.getModule().getIndex(), 
                        MainFrame._INSERTTAB,
                        new String[] {child.getID()}));
            	dco.addChild(child);
            }
        }
        return dco;
    }      
    
    /**
     * Saves the give rows to the database. Saving is done threaded and
     * can be cancelled by the user
     * @see SavingThread
     * @param rows row indices / to be saved rows
     */
    public void save(int[] rows) {
        if (rows.length > 0) {
        	if (!isTaskRunning()) {
                task = new SavingThread(rows, getClass());
                task.start();
            } else {
                new MessageBox(DcResources.getText(Repository.messages.msgJobRunning),
                               MessageBox._WARNING);    
            }
        } else {
            new MessageBox(DcResources.getText(Repository.messages.msgNoItemsToSave), MessageBox._WARNING);    
        }
    }
    
    /**
     * Saves all the new values (including children)
     */
    public void saveNewValues() {
        int rowCount = table.getRowCount();
        int[] rows = new int[rowCount];
        for (int i = 0; i < rowCount; i++) {
        	rows[i] = i;
        }
        save(rows);
    }
    
    /**
     * Saves the selected values only
     * @param silent indicates wether messages should be shown
     */
    public void saveSelectedValues(boolean silent) {
        cancelEdit();
        int[] rows = table.getSelectedRows();
        save(rows);
    }    
    
    public String getParentID() {
    	return parentID;
    }
    
    public void removeRows(String[] ids) {
        if (hasChild()) {
            childPanel.clearTable();
            childPanel.setParentID(null);
        }
        super.removeRows(ids);
    }    
    
    protected abstract Collection getChildren(String parentID);
    public abstract JComponent[] getExtraComponents(); 
    
    /**
     * Creates the panel
     */
    private void buildPanel() {
        JPanel panelInput  = new JPanel(false);
        JPanel panelStatus = getStatusPanel();

        
        //**********************************************************
        //Input panel
        //**********************************************************
        panelInput.setLayout(Layout.getGBL());
 
        table = ComponentFactory.getDCTable(getModule(), false, false);
        table.addMouseListener(tableMouseListener);
        
        JScrollPane scrollResults = new JScrollPane(table);
        scrollResults.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

        panelInput.add( scrollResults,      Layout.getGBC( 0, 0, 7, 1, 20.0, 20.0
                       ,GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                        new Insets(5, 5, 5, 5), 0, 0));
        
        if (hasChild()) {
            childPanel = getChildModule().getInsertPanel();
            
            table.getSelectionModel().addListSelectionListener(new RowSelectionListener());
            table.addKeyListener(new ChangeRowByKey());
            
            JScrollPane scrollResults2 = new JScrollPane(getChildModule().getInsertPanel().table);
            scrollResults2.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
            
            panelInput.add( scrollResults2,   Layout.getGBC( 0, 1, 7, 1, 20.0, 20.0
                           ,GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                            new Insets(5, 5, 5, 5), 0, 0));
        }

        //**********************************************************
        //Main panel
        //**********************************************************
        setLayout(Layout.getGBL());

        add(panelInput,     Layout.getGBC( 0, 0, 8, 1, 120.0, 120.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                            new Insets( 5, 5, 5, 5), 0, 0));
        add(panelTableActions,Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets( 5, 5, 5, 5), 0, 0));
        add(panelStatus,    Layout.getGBC( 0, 2, 8, 1, 1.0, 1.0
                           ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                            new Insets(0, 5, 0, 5), 0, 0));

        ToolTipManager.sharedInstance().registerComponent(table);
    }
    
    // ****************************************************************************
    // ACTIONS
    // ****************************************************************************

    /**
     * @author Nuclear Vision
     * @since 1.4
     * @version 1.5
     */
    private class SavingThread extends DataTask {
        public SavingThread(int[] rows, Class clazz) {
            super(rows, clazz);
        }
        
        public void run() {
        	try {
	            updateProgressBar(0);
	            initProgressBar(rows.length);
	            setStatus("Saving " + rows.length + " items to the database");
	            table.setCursor(new Cursor(Cursor.WAIT_CURSOR));
	            
                if (hasChild()) {
                    childPanel.setParentID(null);
                }
                
                
	            denyActions();
	            startRunning();
	            cancelEdit();
                
	            if (rows.length == 0) {
	                new MessageBox(DcResources.getText(Repository.messages.msgNoChangesToSave), 
	                               MessageBox._INFORMATION);
	                stopRunning();
	            }             
	            
	            boolean ignoreErrors = false;
	            int counter = 1;
	            for (int i = rows.length - 1; i > -1 && keepOnRunning(); i--) {
	                updateProgressBar(counter++);
	                DcObject dco = getObjectForSaving(rows[i]);
	                dco.setRow(rows[i]);
	                if (i == 0) {
                        dco.addRequest(
                                new StatusUpdateRequest(dco.getModule().getIndex(),
                                                        MainFrame._INSERTTAB,
                                                        Repository.messages.msgDataSaved));
	                    dco.setEndOfBatch(true);
	                }
	                
	                try {
	                    dco.saveNew();
	                } catch (ValidationException vExp) {
	                    if (!ignoreErrors) {
	                        SaveQuestionBox question = new SaveQuestionBox(vExp);
	                        if (question.getResult() == SaveQuestionBox._CANCEL) {
	                            stopRunning();
	                        } else if (question.getResult() == SaveQuestionBox._IGNORE) {
	                            ignoreErrors = true;
	                        }
	                    }
	                }
	                
	                try {
	                    sleep(300);
	                } catch (Exception ignore) {} // doesn't want to sleep..                
	            }
        	} finally {
        		table.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        		allowActions();
        		stopRunning();
                
                if (hasChild() && table.getRowCount() > 0) {
                	table.setSelectedRow(0);
                    childPanel.setParentID(table.getValueAt(0,0).toString());
                }
        	}
        }
    }
    
    private class ChangeRowByKey implements KeyListener {
        
        private int rowFrom = -1;
        private int rowTo = -1;
        
        public void keyPressed(KeyEvent e) {
           if (e.getKeyCode() == KeyEvent.VK_UP || e.getKeyCode() == KeyEvent.VK_DOWN) {
           	   rowFrom = table.getSelectedRow();
           }
        }
        
		public void keyReleased(KeyEvent e) {
            if (e.getKeyCode() == KeyEvent.VK_UP || e.getKeyCode() == KeyEvent.VK_DOWN) {
    			rowTo = table.getSelectedRow();
                
                if (rowTo != rowFrom) {
                    String idTo = table.getValueAt(rowTo, 0, true).toString();
                    childPanel.setParentID(idTo);
                } else if (table.getRowCount() == 1) {
                    String id = "" + table.getValueAt(0, 0, true);
                    childPanel.setParentID(id);
                }
            }
        }
        
		public void keyTyped(KeyEvent arg0) {}
    }
    
    private class RowSelectionListener implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent e){
            try {
                if (hasChild()) {
                    String idTo = "";
                    if (e.getFirstIndex() == table.getSelectedRow()) {
                        idTo = "" + table.getValueAt( e.getFirstIndex(), 0, true);
                    } else {
                        idTo = "" + table.getValueAt( e.getLastIndex(), 0, true);
                    }
                    childPanel.setParentID(idTo);
                } 
            } catch (Exception ignore) {}
        }
    }    
    
    /**
     * Saves all selected values 
     * @see TableInsertPanel#saveSelectedValues(boolean)
     */
    private class SaveSelectedValuesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            cancelEdit();
            saveSelectedValues(false);
        }
    }  

    /**
     * Saves all new values (insert action)
     * @see TableInsertPanel#saveNewValues()
     */
    private class SaveNewValuesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveNewValues();
        }
    }    
    
    /**
     * Starts the Amazon update job
     */
    private class AmazonUpdateAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startAmazonUpdate();
        }
    }

    /**
     * Starts the IMDB update job
     */
    private class ImdbUpdateAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startIMDBUpdate();
        }
    }      
}
