/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.expert;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.RandomAccessFile;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Collection;
import java.util.Enumeration;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.ToolTipManager;
import javax.swing.table.TableColumn;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.panels.TablePanel;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.wf.WorkFlow;
import net.sf.ext.util.Converter;

/**
 * Thru this panel any kind of query can be run directly on the database.
 * This panel makes no use of the QueryQueue, the queries are run directly.
 * Queries which have been run successfully are stored in a file and added
 * to the listbox
 *
 * @author Robert Jan van der Waals
 * @since 1.3
 * @version 1.6
 */
public class QueryPanel extends TablePanel {

    private JEditorPane textInput;
	private JButton buttonRunSql;
	private JButton buttonClear;

	private JComboBox comboSQLCommands;

	private WorkFlow wf = new WorkFlow();

    /**
	 * Initializes and creates the panel
	 */
    public QueryPanel() {
        super(null, null, null);
        setHelpIndex("dc.Tools.QueryTool");
        buildPanel();
    }

    /**
	 * Fills the table with the resuls of the query. Headers are read from the
	 * ResultSet and applied on the headers of result table
	 * @param result   the resultset of which the data should be 
     *                 shown in the result table
	 */
    private void fillTable(ResultSet result) throws Exception {
        clearTable();

        int columns = result.getMetaData().getColumnCount();
        
        
        table.setColumnCount(columns);
        int counter = 1;
        for (Enumeration enumerator = table.getColumnModel().getColumns(); enumerator.hasMoreElements(); counter++) {
            TableColumn column = (TableColumn) enumerator.nextElement();
            column.setHeaderValue(result.getMetaData().getColumnName(counter).toLowerCase());
        }

        Object[] values = new String[columns];
        while (result.next()) {
            for (int i = 0; i < columns; i++) {
                values[i] = result.getString((result.getMetaData().getColumnName(i + 1)));    
            }
            table.addRow(values);
        }

        // close the result set
        try {
            result.close();
        } catch (SQLException sqlExp) {
            Messages.add(sqlExp);
        }
        
        // apply the correct headers (colors and such)
        table.applyHeaders();
        setStatus(DcResources.getText(Repository.messages.msgSQLTableHasBeenFilled));
    }
    
    public DcObject getObjectForRow(int row) {
    	return null;
    }
    
    public void openItemForm() {}
    public void createPdfReport() {}
    public void addObjects(Collection objectss) {}
    
    /**
     * Runs the specified query directly on the database, without 
     * making use of the QueryQueue. If an error occurs, it shown 
     * to user in a messagebox.
     * If the query is empty (no query was entered in the screen), 
     * a messagebox is shown is well.
     * If the query was successfull, the query itself is stored 
     * in the query listbox for later use
     */
    protected void runQuery() {
        String sQuery   = textInput.getText().trim();

        if (sQuery.trim().equals("")){
            new MessageBox( DcResources.getText(Repository.messages.msgNoInput), MessageBox._WARNING);
            return;
        }

        try {
        	ResultSet result = wf.runQueryDirectUnclosed(sQuery);

            boolean empty = false;
            if (result != null) {
                try {
                    result.isLast();
                } catch (Exception exp) {
                	empty = true;
                }
            }
            
            if (empty) {
                new MessageBox(DcResources.getText(Repository.messages.msgQueryWasSuccessFull), 
                               MessageBox._INFORMATION);                
            } else {
                fillTable(result);
            } 
            
            addQueryToComboBox(sQuery);
            
            saveDataToFile();            
        } catch (Exception exp) {
            Messages.add(exp);
            new MessageBox(exp.toString(), MessageBox._ERROR);
        }
    }

    /**
     * Adds a query to the bottom of the query combobox
     */
    private void addQueryToComboBox(String sNewQuery) {
        sNewQuery = Converter.deleteEntersFromString(sNewQuery.trim());

        boolean found = false;
        for (int i = 0; i < comboSQLCommands.getItemCount(); i++) {
            Object o = comboSQLCommands.getItemAt(i);
            if (o != null && o instanceof QueryObject) {                    
            	String s = ((QueryObject) o).getQryString();
            	if (s.toLowerCase().equals(sNewQuery.toLowerCase())) {
            		found = true;
            		break;
                }
            }
        }
        if (!found) {
            QueryObject o = new QueryObject(sNewQuery);
            comboSQLCommands.addItem(o);
        }
    }

    /**
     * Fills the query combobox by reading the query file
     */
    private void fillQueryComboBox() {
        File queryFile = new File(DataCrow.baseDir + "data/data_crow_queries.txt");

        try {
            if (queryFile.exists()) {
                RandomAccessFile fileAccess  = new RandomAccessFile(queryFile, "rw");
                long filePointer = 0;
                long fileLength  = queryFile.length();
                comboSQLCommands.addItem("");
                while (filePointer < fileLength) {
                    String query = fileAccess.readLine().trim();
                    addQueryToComboBox(query);
                    filePointer = fileAccess.getFilePointer();
                }
                fileAccess.close();
            } else if (!queryFile.exists()) {
                Messages.add(DcResources.getText(Repository.messages.msgFileNotFound, queryFile.toString()), 
                                    Messages._STANDARD);
                RandomAccessFile fileAccess  = new RandomAccessFile(queryFile, "rw");
                fileAccess.write(Repository.database._PREDEFINEDQRY.getBytes());
                fileAccess.close();
                fillQueryComboBox();
            }
        } catch (Exception exp) {
            comboSQLCommands.addItem("");
        	Messages.add(exp);
        }
        saveDataToFile();
    }

    /**
     * Saves the content of the query combobox to the query file
     */
    protected void saveDataToFile() {
        File queryFile = new File(DataCrow.baseDir + "data/data_crow_queries.txt");
        if (queryFile.exists()) {
            queryFile.delete();
            queryFile = new File(DataCrow.baseDir + "data/data_crow_queries.txt");
        }

        try {
            queryFile.createNewFile();
            RandomAccessFile access  = new RandomAccessFile(queryFile, "rw");
            
            for (int i = 0; i < comboSQLCommands.getItemCount(); i++) {
                Object o = comboSQLCommands.getItemAt(i);
                if (o != null && o instanceof QueryObject) {                    
                    String query = ((QueryObject) o).getQryString();
                    query = Converter.deleteEntersFromString(query);
                    access.writeBytes(query + '\n');
                }
            }
        } catch (Exception exp) {
            Messages.add(
                DcResources.getText(Repository.messages.msgFileSaveError, queryFile.toString()), 
                Messages._ERROR);
            Messages.add(exp);
        }
        Messages.add(
            DcResources.getText(Repository.messages.msgFileSaved, queryFile.toString()), 
            Messages._STANDARD);
    }

    /**
     * Transfers a selected query from the query combobox to the query input field
     */
    protected void setQuery() {
        Object o = comboSQLCommands.getSelectedItem();
        if (o != null) {
            if (o instanceof QueryObject) {
            	QueryObject qry = (QueryObject) o ;
            	textInput.setText(qry.getQryString());
            }
        }
    }

    /**
     * Clears all the values in the panel; the query input field and the result table
     */
    protected void clearPanel() {
        textInput.setText("");
        clearTable();
    }

    /**
     * Clears the table of all values by removing all columns and rows
     */
    public void clearTable() {
        table.clearTable();
    }

    /**
     * Creates the panel
     */
    private void buildPanel() {
        JPanel panelStatus = getStatusPanel();

        //**********************************************************
        //SQL panel
        //**********************************************************
        JPanel panelSQL = new JPanel(false);
        panelSQL.setLayout(Layout.getGBL());

        textInput = ComponentFactory.getEditorPane();

        JScrollPane scrollIn = new JScrollPane(textInput);
        scrollIn.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollIn.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        panelSQL.add(scrollIn, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                     new Insets(0, 5, 5, 5), 0, 0));

        //**********************************************************
        //SQL panel
        //**********************************************************
        JPanel panelQueries = new JPanel(false);
        panelQueries.setLayout(Layout.getGBL());

        comboSQLCommands = ComponentFactory.getComboBox();
        comboSQLCommands.addItemListener(new PredefineQueryAction());
        fillQueryComboBox();

        panelQueries.add(comboSQLCommands, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                         new Insets(5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Result panel
        //**********************************************************
        JPanel panelResult = new JPanel(false);
        panelResult.setLayout(Layout.getGBL());
        table = ComponentFactory.getDCTable(true, false);

        JScrollPane scrollOut = new JScrollPane(table);
        scrollOut.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollOut.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
 
        panelResult.add(scrollOut, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Action panel
        //**********************************************************
        JPanel panelActions = new JPanel(false);
        panelActions.setLayout(Layout.getGBL());

        buttonRunSql = ComponentFactory.getButton(DcResources.getText(Repository.labels.lblRun));
        buttonClear = ComponentFactory.getButton(DcResources.getText(Repository.labels.lblClear));

        buttonRunSql.setMnemonic(KeyEvent.VK_R);
        buttonClear.setMnemonic(KeyEvent.VK_C);

        buttonRunSql.addActionListener(new QueryButtonAction());
        buttonClear.addActionListener(new ClearButtonAction());
        buttonRunSql.setToolTipText(DcResources.getText(Repository.tooltips.tpRunSQL));
        buttonClear.setToolTipText(DcResources.getText(Repository.tooltips.tpClear));

        panelActions.add(buttonRunSql, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                         new Insets(0, 5, 0, 5), 0, 0));
        panelActions.add(buttonClear, Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                         new Insets(0, 0, 0, 5), 0, 0));

        //**********************************************************
        //Main panel
        //**********************************************************
        setLayout(Layout.getGBL());

        panelSQL.setBorder(ComponentFactory.getTitleBorder(
                                DcResources.getText(Repository.labels.lblQueryIn)));
        panelResult.setBorder(ComponentFactory.getTitleBorder(
                                DcResources.getText(Repository.labels.lblQueryOut)));
        panelQueries.setBorder(ComponentFactory.getTitleBorder(
                                DcResources.getText(Repository.labels.lblStoredSQL)));                           

        add(  panelSQL,     Layout.getGBC( 0, 0, 10, 1, 10.0, 10.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                            new Insets(5, 5, 5, 5), 0, 0));
        add(  panelQueries, Layout.getGBC( 0, 1, 10, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                            new Insets(5, 5, 5, 5), 0, 0));
        add(  panelResult,  Layout.getGBC( 0, 2, 10, 1, 20.0, 20.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                            new Insets(5, 5, 5, 5), 0, 0));
        add(  panelActions, Layout.getGBC( 0, 3, 1, 1,  1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets(5, 5, 5, 5), 0, 0));
        add(  panelStatus,  Layout.getGBC( 0, 4, 10, 1, 1.0, 1.0
                           ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                            new Insets(0, 5, 0, 5), 0, 0));

        ToolTipManager.sharedInstance().registerComponent(table);
    }



    // ****************************************************************************
    // INNER CLASSES
    // ****************************************************************************
    
    /**
     * Simple class to represent a query. It shortens the query display string 
     * so it will always fit in the queries combobox 
     */
    private class QueryObject {
        
        private final String query;
        
        public QueryObject(String query) {
            this.query = query;
        }
        
        public String getQryString() {
            return query;
        }
        
        public String toString() {
            if (query.length() > 50) {
                return query.substring(0, 50) + "...";
            } else {
                return query;
            }
        }
    }
    
    private class QueryButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            runQuery();
        }
    }

    private class ClearButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            clearPanel();
        }
    }

    private class PredefineQueryAction implements ItemListener {
        public void itemStateChanged(ItemEvent e) {
            setQuery();
        }
    }
}