/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.forms;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcEditableComboBox;
import net.sf.dc.console.components.DcRatingComboBox;
import net.sf.dc.console.components.panels.TableSearchPanel;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.db.QueryOptions;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.DcFieldDefinitions;
import net.sf.dc.wf.requests.FillTableRequest;

/**
 * Creates a panel for creating the search condition. The search condition exists 
 * out of fields with a value. On this combination will be searched... 
 * Hidden fields are not shown in this panel. Also support searching parent items
 * based on a condition on a child item
 * 
 * @author Robert Jan van der Waals
 * @since 1.5
 * @version 1.9.5
 */
public class SearchForm extends DcDialog {

    private final JComboBox[] fields;
    private final JLabel[] labels;
    
    private final DCModule[] modules;
    private final TableSearchPanel parent;
    private int[] moduleIndices;
    
    private final JPanel panelTech = new JPanel();
    private final JPanel panelInfo = new JPanel();
    
    private final SearchConditionPanel panelConditions;
    private final SearchOptionsPanel panelOptions;
    
    /**
     * Initializes this panel and set the correct fields for searching.
     * Hidden fields are not shown as well notsearchable fields.
     * In case of a child module, the fields of both the parent and the child
     * are shown.
     * @param module the module for which this panel is created
     */
    public SearchForm(DCModule module, TableSearchPanel parent) {
        super(DataCrow.mainFrame);
        setTitle(DcResources.getText(Repository.labels.lblSearchForm));
        setHelpIndex("dc.Searching");
        
        this.parent = parent;
        
        if (module != null && module.getChildModule() != null) {
            modules = new DCModule[2];
            modules[0] = module;
            modules[1] = module.getChildModule();
            
            fields = new JComboBox[module.getFieldCount() + 
                                   module.getChildModule().getFieldCount()];
            labels = new JLabel[   module.getFieldCount() + 
                                   module.getChildModule().getFieldCount()];
        } else if (module != null) {
        	modules = new DCModule[1];
            modules[0] = module;
            fields = new JComboBox[module.getFieldCount()];
            labels = new JLabel[   module.getFieldCount()];
        } else {
            modules = new DCModule[0];
            fields = new JComboBox[0];
            labels = new JLabel[0];
        }

        panelConditions = new SearchConditionPanel();
        panelOptions = new SearchOptionsPanel(module);
        
        buildForm(); 
        
        setBackground(DcSettings.getValueAsColor(Repository.settings.stColor));
        
        setCenteredLocation();
        setModal(true);
    }
    
    public void setVisible(boolean b) {
        if (b) 
            panelConditions.applySettings();
        
        super.setVisible(b);
    }    
    
    /**
     * Searches for the Software objects by using the conditions and options
     * as specified.
     */
    public void searchValues() {
        parent.checkForChanges();
        
        DcObject dco = panelConditions.getSearchValues();
        dco.addRequest(new FillTableRequest(dco.getModule().getIndex()));
        dco.load(panelOptions.getQueryOptions());
        parent.setStatus(DcResources.getText(Repository.messages.msgSearchHasBeenExecuted));
        
        if (parent.hasChild()) {
            parent.getChildModule().getSearchPanel().clearTable();
        }
        
        addToHistory();
        
        DataCrow.mainFrame.setSelectedTab(0);
    }     
    
    private void resetConditions() {
        for (int i = 0; i < fields.length; i++) {
            JComboBox comp = fields[i];
            comp.setSelectedIndex(0);
            comp.getEditor().setItem("");
            if (comp instanceof DcEditableComboBox) {
                DcEditableComboBox combo = (DcEditableComboBox) comp;
                combo.storeValue();
            }
        }         
    }

    private void addToHistory() {
        for (int i = 0; i < fields.length; i++) {
            JComboBox comp = fields[i];
            if (comp instanceof DcEditableComboBox) {
                DcEditableComboBox combo = (DcEditableComboBox) comp;
                combo.storeValue();
            }
        }
    }
    
    private void buildForm() {
        getContentPane().setLayout(Layout.getGBL());
        getContentPane().add( panelConditions, Layout.getGBC( 0, 0, 2, 1, 2.0, 2.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
							  new Insets(5, 5, 5,  5), 0, 0));
        
        getContentPane().add( panelOptions, Layout.getGBC( 0, 1, 2, 1, 2.0, 2.0
        		             ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
							  new Insets(5, 5, 5,  5), 0, 0));
 
        JPanel panelActions = new JPanel();
        JButton buttonSearch = ComponentFactory.getButton(DcResources.getText(Repository.labels.lblFind));
        buttonSearch.setToolTipText(DcResources.getText(Repository.tooltips.tpSearch));
        buttonSearch.addActionListener(new SearchButtonAction());
        buttonSearch.requestFocus();
 
        JButton buttonClose = ComponentFactory.getButton(DcResources.getText(Repository.labels.lblClose));
        buttonClose.addActionListener(new CloseButtonAction());

        JButton buttonReset = ComponentFactory.getButton(DcResources.getText(Repository.labels.lblReset));
        buttonReset.addActionListener(new ResetButtonAction());
        
        panelActions.add(buttonSearch);
        panelActions.add(buttonReset);
        panelActions.add(buttonClose);
        
        getContentPane().add( panelActions, Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                             ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                              new Insets( 5, 5, 5, 5), 0, 0));
        
        pack();     
    }

    /**
     * The search criteria can be defined here  
     */
    private class SearchConditionPanel extends JPanel {
        
        private boolean technicalInfoPresent = false;
        
        public SearchConditionPanel() {
            buildDialog();         
        }
        
        /**
         * Returns the values on which will be searched (the search condition)
         */
        public DcObject getSearchValues() {
            DcObject[] objects = new DcObject[modules.length]; 
            
            int diff = 0;
            for (int j = 0; j < modules.length; j++) {
                DcObject dco = modules[j].getDCObject();
                
                Object[] values = dco.getValues();
                int[] indices = dco.getFieldIndices();
                for (int i = 0; i < indices.length; i++) {
                    JComboBox component = fields[indices[i]  + diff];
                    values[indices[i]] = component.getSelectedItem();
                }
                
                for (int i = 0; i < values.length; i++) {
                    if (dco.isSearchable(i)) {
                        String s = values[i] == null ? "" : values[i].toString();
                        if (s != null && !s.equals("")) {
                            dco.setValue(i, values[i]);
                        }
                    }
                }
                
                objects[j] = dco;
                diff = diff + dco.getFields().length;
            }
            
            if (objects.length > 1) {
                objects[0].addChild(objects[1]);
            }
            return objects[0];
        }
        
        protected void createFields() {
            int diff = 0;
            for (int i = 0; i < modules.length; i++) {
                DcFieldDefinitions definitions = 
                    (DcFieldDefinitions) DcSettings.getValueAsDefinitionGroup(modules[i].getFieldDefinitionsSettingsKey());
                DcObject dco = modules[i].getDCObject();
                
                for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
                    DcFieldDefinition definition = (DcFieldDefinition) iter.next();
                    int field = definition.getIndex();
                    
                    JLabel label = ComponentFactory.getLabel(dco.getLabel(field));
                    JComponent input = ComponentFactory.getComponent(
                                                            dco.getFieldType(field), 
                                                            dco.getLabel(field),
                                                            dco.getMaxFieldLength(field));
                    
                    input.setPreferredSize(new Dimension(200,20));
                    label.setPreferredSize(new Dimension(150,20));
                    
                    if (!(input instanceof JComboBox)) {
                        input = new DcEditableComboBox();
                    } else {
                        JComboBox combo = (JComboBox) input;
                        if (combo instanceof DcRatingComboBox) {
                            combo.addItem(DcResources.getText(Repository.labels.lblEmptySearchValue));
                        } else {
                            combo.insertItemAt(DcResources.getText(Repository.labels.lblEmptySearchValue), 1);
                        }
                    }
                    
                    labels[field + diff] = label;
                    fields[field + diff] = (JComboBox) input;
                }
                
                diff = diff + dco.getFields().length;
                
                // not the real module index, but the index for the modules array
                moduleIndices[i] = i;                
            }
        }
            
        protected void applySettings() {
            panelInfo.removeAll();
            panelTech.removeAll();
            
            int yTech = 0;
            int yInfo = 0;
            
            int diff = 0;
                    
            for (int i = 0; i < modules.length; i++) {
                DCModule module = modules[i];
                DcFieldDefinitions definitions = (DcFieldDefinitions) DcSettings.getValueAsDefinitionGroup(
                                                            module.getFieldDefinitionsSettingsKey());
            
                DcObject dco = module.getDCObject();
                
                for (Iterator iter = definitions.iterator(); iter.hasNext(); ) {
                    DcFieldDefinition definition = (DcFieldDefinition) iter.next();
                    
                    int index = definition.getIndex();
                    JComboBox input = fields[definition.getIndex() + diff];
                    JLabel label = labels[definition.getIndex()  + diff];
                    label.setText(dco.getLabel(definition.getIndex()));
                    
                    if (definition.isEnabled() && dco.isSearchable(index)) {
                        if (!dco.getField(index).isTechnicalInfo()) {
                            panelInfo.add(label, Layout.getGBC(0, yInfo, 1, 1, 1.0, 1.0
                                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                      new Insets(0, 0, 0, 5), 0, 0));
                            panelInfo.add(input,  Layout.getGBC(1, yInfo, 1, 1, 1.0, 1.0
                                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                      new Insets(0,  5, 0, 5), 0, 0));
                            yInfo++;
                        } else {
                            technicalInfoPresent = true;
                            panelTech.add(label, Layout.getGBC(0, yTech, 1, 1, 1.0, 1.0
                                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                     new Insets(0, 0, 0, 5), 0, 0));
                            panelTech.add(input,  Layout.getGBC(1, yTech, 1, 1, 1.0, 1.0
                                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                     new Insets(0,  5, 0, 5), 0, 0));
                            yTech++;
                        }
                    }
                }
                diff = diff + dco.getFields().length;
            }
        }
        
        /**
         * Creates the panel
         */
        private void buildDialog() {
            setLayout(Layout.getGBL());
            Color color = DcSettings.getValueAsColor(Repository.settings.stColor);
            
            int length = 0;
            for (int i = 0; i < modules.length; i++) {
                length += modules[i].getFieldCount();
            }
            
            moduleIndices = new int[length];

            panelInfo.setLayout(Layout.getGBL());
            panelInfo.setBackground(color);
            panelTech.setLayout(Layout.getGBL());
            panelTech.setBackground(color);
            
            createFields();
            applySettings();
            
            JTabbedPane tabbedPane = new JTabbedPane();
            tabbedPane.addTab(DcResources.getText(Repository.labels.lblDescription), IconLibrary._icoDescription, panelInfo);
            if (technicalInfoPresent) {
                JPanel dummy = new JPanel();
                dummy.setLayout(Layout.getGBL());
                dummy.setBackground(color);
                dummy.add(panelTech,  Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                          ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets(2, 0, 0, 0), 0, 0));
                tabbedPane.addTab(DcResources.getText(Repository.labels.lblTechnicalInfo), IconLibrary._icoTechInfo, dummy);
            }  
            
            add(tabbedPane, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                    new Insets(0, 0, 0, 0), 0, 0));
        }   
    }
    
    /**
     * Search Option panel. Here options, such as 
     * comply to all conditions, are maintained
     */
    public class SearchOptionsPanel extends JPanel {
        private JCheckBox checkComplyAll = 
            ComponentFactory.getCheckBox(DcResources.getText(Repository.labels.lblAndOrSelect));
        private JCheckBox checkPrecise =
            ComponentFactory.getCheckBox(DcResources.getText(Repository.labels.lblPreciseSelect));
        private JLabel labelOrderBy = ComponentFactory.getLabel(
                                      DcResources.getText(Repository.labels.lblOrderBy));
        private JComboBox comboOrderBy = ComponentFactory.getComboBox();

        private DcObject dco;
        
        public SearchOptionsPanel(DCModule module) {
            dco = module.getDCObject();
            setLayout(Layout.getGBL());

            String[] colNames    = dco.getLabels();
            comboOrderBy.addItem("");
            for (int i = 0; i < colNames.length; i++) {
                comboOrderBy.addItem(colNames[i]);
            }
            
            comboOrderBy.setSelectedItem(colNames[1]);
            checkComplyAll.setSelected(true);
            
            add( checkComplyAll, Layout.getGBC( 0, 0, 2, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 3, 5), 0, 0));
            add( checkPrecise,   Layout.getGBC( 0, 1, 2, 1, 5.0, 5.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 3, 5), 0, 0));
            add( labelOrderBy,   Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 0, 5), 0, 0));
            add( comboOrderBy,   Layout.getGBC( 1, 2, 1, 1, 5.0, 5.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                 new Insets( 0, 5, 5, 5), 0, 0));
        }

        /**
         * Returns the query options used for searching values
         */
        public QueryOptions getQueryOptions() {
            boolean bPreciseSelect = checkPrecise.isSelected();
            boolean bComplyToAllCondition = checkComplyAll.isSelected();
            String sOrderBy = "";
            if (comboOrderBy.getSelectedIndex() > 0) {
                sOrderBy = dco.getModule().getField(comboOrderBy.getSelectedIndex() - 1).getDatabaseFieldName();
            }
            return new QueryOptions(sOrderBy, bComplyToAllCondition, bPreciseSelect);
        }
    }  

    
    public class ResetButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            resetConditions();
        }
    } 
    
    public class CloseButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            setVisible(false);
        }
    } 
    
    public class SearchButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            searchValues();
        }
    }    
}
