/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.core.Repository;

/**
 * Definition of a field of a Data Crow Object.
 * The label, length, value type, etc is described here
 * 
 * @author Robert Jan van der Waals
 * @since 1.3
 * @version 1.0
 */
public class DcField {
    
    private String systemName;
    private String label;
    private boolean enabled;
    private boolean required;
    private boolean hidden;
    private boolean searchable;
    private boolean readonly;
    private boolean techinfo;
    private int maximumLength;
    private int fieldType;
    private int valueType;
    private int index;
    private String databaseFieldName;
    
    /**
     * Creates a new field definition. 
     * 
     * @param label display value
     * @param readonly writable or not
     * @param maximumLength maximum inout length
     * @param fieldType UI component type (@see ComponentFactory)
     * @param valueType input value type (@see Repository.datatypes)
     * @param databaseFieldName database column name
     */
    public DcField( int index,
                    String label,  
                    boolean enabled,
                    boolean readonly,
                    boolean searchable,
                    boolean techinfo, 
                    int maximumLength,
                    int fieldType,
                    int valueType,
                    String databaseFieldName) {
        
        setEnabled(enabled);
        setIndex(index);
        setLabel(label);
        setSystemName(label);
        setReadOnly(readonly);
        setSearchable(searchable);
        setTechinfo(techinfo);
        setMaximumLength(maximumLength);
        setFieldType(fieldType);
        setValueType(valueType);
        setDatabaseFieldName(databaseFieldName);
    }
    
    public void setIndex(int index) {
        this.index = index;
    }
    
    public int getIndex() {
        return index;
    }
    
    /**
     * Can there be searched on this field ?
     * @param searchable 
     */
    public void setTechinfo(boolean techinfo) {
        this.techinfo = techinfo;   
    }
    
    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }
    
    public boolean isEnabled() {
        return enabled;
    }
    
    /**
     * Can there be searched on this field ?
     * @param searchable 
     */
    public void setSearchable(boolean searchable) {
        this.searchable = searchable;   
    }
    
    /**
     * Returns the database field name (used for queries)
     * @return correct database field name
     */
    public String getDatabaseFieldName() {
        return databaseFieldName;
    }
    
    /**
     * Returns the UI component type (@see ComponentFactory)
     */
    public int getFieldType() {
        return fieldType;
    }
    
    /**
     * Returns the input value type (@see Repository.datatypes)
     */
    public int getValueType() {
        return valueType;
    }

    /**
     * Returns the display value
     */
    public String getLabel() {
        return label;
    }
    
    /**
     * Returns the system name for this field (the original name)
     */
    public String getSystemName() {
        return systemName;
    }    
    
    /**
     * Indicates whether this field is visible or not within
     * the UI 
     */
    public boolean isHidden() {
        return hidden;
    }
    
    /**
     * Is this field required (must be filled)
     * @return is required y/n
     */
    public boolean isTechnicalInfo() {
        return techinfo;
    }
    
    /**
     * Is this field required (must be filled)
     * @return is required y/n
     */
    public boolean isRequired() {
        return required;
    }
    
    /**
     * Is this field readonly (like the ID fields)
     * @return is readonly y/n
     */
    public boolean isReadOnly() {
        return readonly;
    }
    
    /**
     * Defines whether a search can be performed on this field.
     * Example; on picture fields cannot be searched
     */
    public boolean isSearchable() {
        return searchable;
    }
    
    /**
     * Returns the maximum input length
     */
    public int getMaximumLength() {
        return maximumLength;
    }

    /**
     * Sets the display value for this field
     */
    public void setLabel(String s) {
        label = s;        
    }
    
    private void setSystemName(String s) {
        systemName = s;        
    }    

    /**
     * Sets this field to uneditable 
     * @param b read only y/n
     */
    public void setReadOnly(boolean b) {
        readonly = b;
    }

    /**
     * Makes this field required
     * @param b required y/n
     */
    public void setRequired(boolean b) {
        required = b;
    }
    
    /**
     * Makes this field hidden (for UI purposes)
     * @param b hidden y/n
     */
    public void setHidden(boolean b) {
        hidden = b;
    }
    
    /**
     * Sets the maximum input length
     * @param i maximum input length
     */
    public void setMaximumLength(int i) {
        maximumLength = i;
    }
    
    /**
     * Sets the UI field type @see ComponentFactory
     * @param index Component type index
     */
    public void setFieldType(int index) {
        fieldType = index;
    }

    /**
     * Sets the input value type @see Repository.datatypes
     * @param index the value type index
     */
    public void setValueType(int index) {
        valueType = index;
    }
    
    /**
     * Sets the database column name (database mapping)
     * @param s database field name
     */
    public void setDatabaseFieldName(String s) {
        databaseFieldName = s;
    }
    
    public String toString() {
        return label;
    }
    
    /**
     * Determines and returns the database field type
     */
    public String getDataBaseFieldType() {
        String s = "";
        if (getValueType() == Repository.datatypes._STRING) {
            if (getFieldType() == ComponentFactory._LONGTEXTFIELD) {
                s = Repository.database._FIELDLONGSTRING + 
                    "(" + getMaximumLength() + ")";
            } else {
                s = Repository.database._FIELDSTRING + 
                    "(" + getMaximumLength() + ")";
            }
        } else if (getValueType() == Repository.datatypes._INTEGER) {
            s = Repository.database._FIELDINTEGER;
        } else if (getValueType() == Repository.datatypes._OBJECT) {
            s = Repository.database._FIELDOBJECT;            
        } else if (getValueType() == Repository.datatypes._TYPEREFERENCE) {
            s = Repository.database._FIELDSTRING + 
                "(" + getMaximumLength() + ")";
        } else if (getValueType() == Repository.datatypes._PARENTREFERENCE) {
            s = Repository.database._FIELDSTRING + 
                "(" + getMaximumLength() + ")";
        }
        return s;
    }
}
