/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.forms.InternetSearchForm;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.book.Book;
import net.sf.dc.http.HttpConnection;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.ext.util.StringUtils;

/**
 * @author Robert Jan van der Waals
 * @version 1.9
 * @since 1.4
 */
public abstract class AmazonUpdate extends InternetUpdateProcess {
    
    protected String searchString;
    protected final boolean broadband;
    protected final InternetSearchForm ui;
    
    public AmazonUpdate(InternetSearchForm ui,
                        String searchString, 
                        boolean broadband) {
        this.ui = ui;
        this.searchString = searchString;
        this.broadband = broadband;
    }

    
    /**
     * Searches on the Amazon.com website on the catagories software and
     * videogames. Returns a Vector of Strings. Each String describes 
     * a found item on the Amazon website
     */
    protected void search(URL url) {
        ui.setCancelMode();
        try {
            ui.addMessage(DcResources.getText(Repository.messages.msgConnectingToServer, 
                          DcSettings.getValueAsString(Repository.settings.stAmazonServer)));            
            HttpConnection connection = new HttpConnection(url);
            String result = connection.getString(url);
            connection.close();
            createObjects(result);
        } catch (Exception e) {
            new MessageBox(e.toString(), MessageBox._ERROR);
            Messages.add(e);
            ui.addMessage("Could not connect to Amazon! " + e.getMessage());
        } finally {
            ui.setSearchMode();
        }
    }
    
    /**
     * Splits one search result in to pieces. Every piece (String)
     * is a presentation of one item (raw data)
     * @param rawResult
     */
    protected void createObjects(String rawResult) {
        Collection parts = new ArrayList();
        int index = rawResult.indexOf("<Details");
        while (index > -1 && !cancelled) {
            int end = rawResult.indexOf("</Details>") + 10;
            if (end != -1) {
                String part = rawResult.substring(index, end);
                parts.add(part);
                rawResult = rawResult.substring(end, rawResult.length());
            }
            index = rawResult.indexOf("<Details");
        }
        
        if (parts.size() == 0) {
            new MessageBox(DcResources.getText(Repository.messages.msgNoResultsForKeywords,
                           searchString), MessageBox._WARNING);
            return;
        }
        
        ui.addMessage(DcResources.getText(Repository.messages.msgFoundXResults, 
                      String.valueOf(parts.size())));          
        ui.addMessage(DcResources.getText(Repository.messages.msgStartParsingXResults, 
                      String.valueOf(parts.size())));         
        
        ui.initProgressBar(parts.size());
        int counter = 1;
        for (Iterator iter = parts.iterator(); iter.hasNext() && !cancelled; ) {
            DcObject dco = convertXmlToObject(iter.next().toString());
            ui.addMessage(DcResources.getText(Repository.messages.msgParsingSuccessfull, 
                          dco.getValue(Book._A_TITLE).toString()));          
            ui.addObject(dco);
            ui.updateProgressBar(counter++);
        }
        parts.clear();
        rawResult = null;
    }   
    
    protected abstract DcObject convertXmlToObject(String s);
    
    protected Integer getRating(String rawResult) {
        int indexBegin = 0;
        int indexEnd = 0;
        
        String rating = "";
        indexBegin = rawResult.indexOf("<AvgCustomerRating>");
        if (indexBegin > -1) {
            indexBegin = indexBegin + 19;
            indexEnd = rawResult.indexOf("</AvgCustomerRating>");
            rating = rawResult.substring(indexBegin, indexEnd);
        }
        
        Integer iRating = new Integer(0);
        try { 
            if (rating.trim().length() > 0) {
                iRating = new Integer(Math.round(Float.valueOf(rating).floatValue()));
            }
        } catch (Exception ignore) {};
        
        return iRating;
    }
    
    protected String getManufacturer(String rawResult) {
        int indexBegin = 0;
        int indexEnd = 0;
        
        String manufacturer = "";
        indexBegin = rawResult.indexOf("<Manufacturer>");
        if (indexBegin > -1) {
            indexBegin = indexBegin + 14;
            indexEnd = rawResult.indexOf("</Manufacturer>");
            manufacturer = rawResult.substring(indexBegin, indexEnd);
        }
        return StringUtils.convertHttpString(manufacturer);
    }
    
    protected Integer getReleaseDate(String rawResult) {
        int indexBegin = 0;
        int indexEnd = 0;
        
        String releaseDate = "";
        Integer releaseYear = new Integer(0);
        indexBegin = rawResult.indexOf("<ReleaseDate>");
        if (indexBegin != -1) {
            indexBegin = indexBegin + 13;
            indexEnd = rawResult.indexOf("</ReleaseDate>");
            releaseDate = rawResult.substring(indexBegin, indexEnd);
            try {
                releaseDate = releaseDate.substring(releaseDate.indexOf(",") + 1, releaseDate.length()).trim();
                releaseYear = new Integer(Integer.parseInt(releaseDate));
            } catch (Exception ignore) {}
        }
        return releaseYear;
    }
    
    /**
     * Creates a description out of several pieces of information
     * @param rawResult the raw input string
     */
    protected String getDescription(String rawResult) {
        int indexBegin = 0;
        int indexEnd = 0;

        String companyDescription = "";
        indexBegin = rawResult.indexOf("<ProductDescription>");
        if (indexBegin != -1) {
            indexBegin = indexBegin + 20;
            indexEnd = rawResult.indexOf("</ProductDescription>");
            companyDescription = rawResult.substring(indexBegin, indexEnd);
        }

        String features = "";
        indexBegin = rawResult.indexOf("<Features>");
        if (indexBegin > -1) {
            indexBegin = indexBegin + 10;
            indexEnd = rawResult.indexOf("</Features>");
            features = rawResult.substring(indexBegin, indexEnd);

            String temp = "";
            indexBegin = features.indexOf("<Feature>");
            int counter = 1;
            while(indexBegin > -1) {
                indexBegin = indexBegin + 9;
                indexEnd = features.indexOf("</Feature>");

                temp += "" + (counter++) + ") " + features.substring(indexBegin, indexEnd) + "\n";

                indexEnd = indexEnd + 10;
                features = features.substring(indexEnd, features.length());
                indexBegin = features.indexOf("<Feature>");
            }
            
            if (temp.trim().length() > 0) {
                features = "-- features --" + "\n" + temp;
            } else {
                features = "";
            }
        }
        String userReviews = getUserReview(rawResult);
        
        String description = "";
        if (companyDescription.trim().length() > 0) {
            description += companyDescription;
        }
        
        if (userReviews.trim().length() > 0) {
            if (description.length() > 0) {
                description += "\n\n";
            }
            description += userReviews;
        }        
        
        if (features.trim().length() > 0) {
            if (description.length() > 0) {
                description += "\n\n";
            }
            description += features;
        }
        
        return StringUtils.convertHttpString(description);
    }
    
    protected String getUserReview(String xmlString) {
        int begin = xmlString.indexOf("<Reviews>");
        int end = xmlString.lastIndexOf("</Reviews>");
        
        String reviews = "";
        if (begin > -1 && end > -1) {
            String xmlReviews = xmlString.substring(begin + 8, end);
            
            int counter = 1;
            while (xmlReviews.indexOf("<Comment>") > -1) {
                if (counter > 1) {
                    reviews += "\n\n";
                }
                begin = xmlReviews.indexOf("<Comment>");
                end = xmlReviews.indexOf("</Comment>"); 
                reviews += "-- review " + (counter++) + " --\n" + 
                           xmlReviews.substring(begin + 9, end);
                xmlReviews = xmlReviews.substring(end + 10, xmlReviews.length());
            }
        }
        return reviews;
    }    
}
