package net.sf.ext.mediamanager.ebm.matroska;

import net.sf.ext.mediamanager.ebml.*;

/**
 * <p>Title: JEBML</p>
 * <p>Description: Java Classes to Read EBML Elements</p>
 * <p>Copyright: Copyright (c) 2004 Jory Stone <jcsston@toughguy.net></p>
 *
 * @author jcsston
 * @version 1.0
 */

public class MatroskaBlock extends BinaryElement {
  protected int [] Sizes = null;
  protected int HeaderSize = 0;
  protected int BlockTimecode = 0;
  protected int TrackNo = 0;

  public MatroskaBlock(byte[] type) {
    super(type);
  }

  public void readData(DataSource source) {
    parseBlock(source);
  }

  public void parseBlock(DataSource source) {

    TrackNo = (int)EBMLReader.readEBMLCode(source);
    HeaderSize += Element.EBMLCodedSizeLength(TrackNo);

    short BlockTimecode1 = (short)(source.readByte() & 0xFF);
    short BlockTimecode2 = (short)(source.readByte() & 0xFF);
    if (BlockTimecode1 != 0 || BlockTimecode2 != 0) {
      BlockTimecode = (BlockTimecode1 << 8) | BlockTimecode2;
    }

    int LaceFlag = source.readByte() & 0x06;
    // Increase the HeaderSize by the number of bytes we have read
    HeaderSize += 3;
    if (LaceFlag != 0x00) {
      // We have lacing
      byte LaceCount = source.readByte();
      HeaderSize += 1;
      if (LaceFlag == 0x02) { // Xiph Lacing
        Sizes = readXiphLaceSizes(source, LaceCount);

      } else if (LaceFlag == 0x06) { // EBML Lacing
        Sizes = readEBMLLaceSizes(source, LaceCount);

      } else if (LaceFlag == 0x04) { // Fixed Size Lacing
        Sizes = new int[LaceCount+1];
        Sizes[0] = (int)(this.getSize() - HeaderSize) / (LaceCount+1);
        for (int s = 0; s < LaceCount; s++)
          Sizes[s+1] = Sizes[0];
      } else {
        throw new java.lang.RuntimeException("Unsupported lacing type flag.");
      }
    }
    data = new byte[(int)(this.getSize() - HeaderSize)];
    source.read(data, 0, data.length);
    this.dataRead = true;
  }

  public int[] readEBMLLaceSizes(DataSource source, short LaceCount) {
    int [] LaceSizes = new int[LaceCount+1];
    LaceSizes[LaceCount] = (int)this.getSize();

    // This uses the DataSource.getBytePosition() for finding the header size
    // because of the trouble of finding the byte size of sized ebml coded integers
    int ByteStartPos = source.getBytePosition();

    LaceSizes[0] = (int)EBMLReader.readEBMLCode(source);
    long FirstEBMLSize = LaceSizes[0];
    long LastEBMLSize = 0;
    for (int l = 0; l < LaceCount-1; l++) {
      LastEBMLSize = EBMLReader.readSignedEBMLCode(source);

      FirstEBMLSize += LastEBMLSize;
      LaceSizes[l+1] = (int)FirstEBMLSize;

      // Update the size of the last block
      LaceSizes[LaceCount] -= LaceSizes[l+1];
    }
    int ByteEndPos = source.getBytePosition();

    HeaderSize = HeaderSize + (ByteEndPos - ByteStartPos);
    LaceSizes[LaceCount] -= HeaderSize;

    return LaceSizes;
  }

  public int[] readXiphLaceSizes(DataSource source, short LaceCount) {
    int [] LaceSizes = new int[LaceCount+1];
    LaceSizes[LaceCount] = (int)this.getSize();

    source.getBytePosition();

    for (int l = 0; l < LaceCount; l++) {
      short LaceSizeByte = 255;
      while (LaceSizeByte == 255) {
        LaceSizeByte = (short)(source.readByte() & 0xFF);
        HeaderSize += 1;
        LaceSizes[l] += LaceSizeByte;
      }
      // Update the size of the last block
      LaceSizes[LaceCount] -= LaceSizes[l];
    }
    
    source.getBytePosition();

    LaceSizes[LaceCount] -= HeaderSize;

    return LaceSizes;
  }

  public int getFrameCount() {
    if (Sizes == null) {
      return 1;
    }
    return Sizes.length;
  }

  public byte [] getFrame(int frame) {
    if (Sizes == null) {
      if (frame != 0) {
        throw new java.lang.IllegalArgumentException("Tried to read laced frame on non-laced Block. MatroskaBlock.getFrame(frame > 0)");
      }
      return data;
    }
    byte [] FrameData = new byte[Sizes[frame]];

    // Calc the frame data offset
    int StartOffset = 0;
    for (int s = 0; s < frame; s++) {
      StartOffset += Sizes[s];
    }

    // Copy the frame data
    System.arraycopy(data, StartOffset, FrameData, 0, FrameData.length);

    return FrameData;
  }
  public long getAdjustedBlockTimecode(long ClusterTimecode, long TimecodeScale) {
    return ClusterTimecode + (BlockTimecode * TimecodeScale);
  }
  public int getTrackNo() {
    return TrackNo;
  }
  public int getBlockTimecode() {
    return BlockTimecode;
  }
}
